package cn.gygxzc.tina.cache.caffeine

import com.github.benmanes.caffeine.cache.Caffeine
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.cache.CacheManager
import org.springframework.cache.caffeine.CaffeineCache
import org.springframework.cache.support.SimpleCacheManager
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import java.util.concurrent.TimeUnit

/**
 * Created by niantuo on 2018/9/29.
 * caffeine 缓存配置
 */

@Configuration
@EnableConfigurationProperties(CaffeineProperties::class)
@ConditionalOnProperty(prefix = "cache.caffeine", name = arrayOf("enable"), havingValue = "true")
class CaffeineConfiguration {

    @Autowired
    private lateinit var properties: CaffeineProperties

    @Bean("caffeineCacheManager")
    fun caffeineCacheManager(): CacheManager {
        return SimpleCacheManager().also {
            it.setCaches(buildCache())
        }
    }

    fun buildCache(): MutableCollection<CaffeineCache> {
        val collection = mutableListOf<CaffeineCache>()
        properties.cacheSpec.forEach { name, spec ->
            run {
                val builder = Caffeine.newBuilder().also {
                    if (spec.initialCapacity > 0) {
                        it.initialCapacity(spec.initialCapacity)
                    }
                    if (spec.maximumSize > 0) {
                        it.maximumSize(spec.maximumSize)
                    }
                    if (spec.maximumWeight > 0) {
                        it.maximumWeight(spec.maximumWeight)
                    }
                    if (spec.expireAfterAccess > 0) {
                        it.expireAfterAccess(spec.expireAfterAccess, TimeUnit.SECONDS)
                    }
                    if (spec.expireAfterWrite > 0) {
                        it.expireAfterWrite(spec.expireAfterWrite, TimeUnit.SECONDS)
                    }
                    if (spec.refreshAfterWrite > 0) {
                        it.refreshAfterWrite(spec.refreshAfterWrite, TimeUnit.SECONDS)
                    }
                    if (spec.isWeakKeys) {
                        it.weakKeys()
                    }
                    if (spec.isWeakValues) {
                        it.weakValues()
                    }
                    if (spec.isSoftValues) {
                        it.softValues()
                    }
                    if (spec.isRecordStats) {
                        it.recordStats()
                    }
                }
                collection.add(CaffeineCache(name, builder.build()))
            }
        }
        return collection
    }
}
