package cn.gygxzc.tina.cache.redis

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.autoconfigure.data.redis.RedisProperties
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.cache.CacheManager
import org.springframework.cache.annotation.EnableCaching
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Primary
import org.springframework.data.redis.cache.RedisCacheManager
import org.springframework.data.redis.connection.RedisConnectionFactory
import org.springframework.data.redis.connection.RedisPassword
import org.springframework.data.redis.connection.RedisStandaloneConfiguration
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory

/**
 * Redis 缓存配置
 * 缓存管理器
 * @author NorthLan
 * @date 2018/3/29
 * @url https://noahlan.com
 */
@Configuration
@EnableCaching
@ConditionalOnProperty(prefix = "spring.cache", name = ["type"], havingValue = "redis")
@EnableConfigurationProperties(RedisProperties::class)
class RedisConfig {

    @Bean
    @Primary
    fun connectFactory(properties: RedisProperties): LettuceConnectionFactory {
        val configuration = RedisStandaloneConfiguration(properties.host, properties.port)
        configuration.database = properties.database
        configuration.password = RedisPassword.of(properties.password)
        return LettuceConnectionFactory(configuration)
    }


    @Bean("redisCacheManager")
    @Primary
    fun cacheManager(redisConnectionFactory: RedisConnectionFactory): CacheManager {
        return RedisCacheManager.create(redisConnectionFactory)
    }

}