package cn.gygxzc.tina.cros

import org.slf4j.LoggerFactory
import org.springframework.http.HttpMethod
import org.springframework.http.HttpStatus
import org.springframework.stereotype.Component
import org.springframework.web.server.ServerWebExchange
import org.springframework.web.server.WebFilter
import org.springframework.web.server.WebFilterChain
import reactor.core.publisher.Mono

/**
 * Created by niantuo on 2018/9/29.
 * 管理跨域问题的filter，正式项目中是不需要的，
 * 因为到业务子系统不会出现options请求了
 */
@Component
class CrosWebFilter : WebFilter {
    private val logger = LoggerFactory.getLogger(CrosWebFilter::class.java)

    override fun filter(exchange: ServerWebExchange, chain: WebFilterChain): Mono<Void> {

        val httpRequest = exchange.request
        val httpResponse = exchange.response
        httpResponse.headers
        httpResponse.headers.apply {
            set("Access-control-Allow-Origin", httpRequest.headers["Origin"])
            set("Access-Control-Allow-Methods", "GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS, PATCH")
            set("Access-Control-Allow-Credentials", "true")
            set("Access-Control-Allow-Headers", httpRequest.headers["Access-Control-Request-Headers"])
            val method = httpRequest.method
            if (method == HttpMethod.POST || method == HttpMethod.PUT) {
                set("Access-Control-Expose-Headers", "Location")
            } else if (method == HttpMethod.OPTIONS) {
                httpResponse.statusCode = HttpStatus.OK
                logger.info("options cros allow {}", httpRequest.path)
                return httpResponse.setComplete()
            }
        }
        logger.info("cros allow {}", httpRequest.path)
        return chain.filter(exchange)
    }
}