package cn.gygxzc.tina.orm

import com.baomidou.mybatisplus.annotation.DbType
import com.baomidou.mybatisplus.generator.AutoGenerator
import com.baomidou.mybatisplus.generator.InjectionConfig
import com.baomidou.mybatisplus.generator.config.*
import com.baomidou.mybatisplus.generator.config.converts.MySqlTypeConvert
import com.baomidou.mybatisplus.generator.config.po.TableInfo
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy
import java.io.File

/**
 * 代码生成器
 * @author NorthLan
 * @date 2018/1/24
 * @url https://noahlan.me
 */
fun main(args: Array<String>) {
    generate(true, "NorthLan", "test", DataSourceConfig().also {
        it.dbType = DbType.MYSQL
        it.typeConvert = MySqlTypeConvert()
        it.driverName = "com.mysql.jdbc.Driver"
        it.username = "archives"
        it.password = "archives"
        it.url = "jdbc:mysql://192.168.1.10:3306/archives_sys?characterEncoding=utf8"
        //        it.url = "jdbc:mysql://127.0.0.1:3306/archives_mgr?characterEncoding=utf8"
        //        it.username = "root"
        //        it.password = "root"
    })
}

/**
 * 代码生成器具体执行方法<br>
 * @param isKotlin 是否生成kotlin代码
 * @param author 作者
 * @param pkgType 包类别 (sys/bus)
 * @param dataSourceConfig 数据源配置
 */
fun generate(isKotlin: Boolean, author: String, pkgType: String, dataSourceConfig: DataSourceConfig) {
    val mpg = AutoGenerator()

    val path = File("zen-api").absolutePath

    // 全局配置
    mpg.globalConfig = GlobalConfig().also {
        it.outputDir = if (isKotlin) {
            "$path/src/main/kotlin"
        } else {
            "$path/src/main/java"
        }
        it.isFileOverride = true
        it.isActiveRecord = false
        it.isEnableCache = false
        it.isOpen = false
        it.author = author
        it.isKotlin = isKotlin
        it.isBaseResultMap = true
        it.isBaseColumnList = false
        it.mapperName = "%sMapper"
        it.xmlName = "%sMapper"
        it.serviceName = "I%sService"
        it.serviceImplName = "%sServiceImpl"
    }

    // 数据源
    mpg.dataSource = dataSourceConfig

    // 策略配置
    mpg.strategy = StrategyConfig().also {
        it.isEntityLombokModel = true
        it.isEntityLombokModel = true
        it.isCapitalMode = false
        it.naming = NamingStrategy.underline_to_camel
//        it.setTablePrefix("t")
        it.superEntityClass = "com.gxzc.zen.common.base.BaseModel"
        it.setSuperEntityColumns("id", "create_by", "create_time", "update_by", "update_time", "remark", "enable")
        it.superMapperClass = "com.gxzc.zen.common.base.IBaseMapper"
        it.superServiceClass = "com.gxzc.zen.common.base.IBaseService"
        it.superServiceImplClass = "com.baomidou.mybatisplus.service.impl.ServiceImpl"
//            superControllerClass = ""
        it.setInclude("sys_upload_info")
        it.isEntityBooleanColumnRemoveIsPrefix = true
        it.logicDeleteFieldName = "enable"
//            setExclude("")
    }

    // 包配置
    mpg.packageInfo = PackageConfig().also {
        it.parent = "com.gxzc.zen.api.$pkgType"
        it.entity = "model"
    }


    val cfg = object : InjectionConfig() {
        override fun initMap() {
        }
    }.also {
        it.fileOutConfigList = ArrayList<FileOutConfig>().also {
            it.add(object : FileOutConfig("/templates/mapper.xml.vm") {
                override fun outputFile(tableInfo: TableInfo?): String {
                    return "$path/src/main/resources/mapping/$pkgType/${tableInfo?.mapperName}.xml"
                }
            })
        }
    }

    mpg.cfg = cfg

    mpg.template = TemplateConfig().also {
        it.controller = null // 不生成controller
        it.xml = null
//        it.service = null
//        it.mapper = null
    }

    mpg.execute()
}


