package cn.gygxzc.tina.orm.base

import com.baomidou.mybatisplus.core.conditions.Wrapper
import com.baomidou.mybatisplus.core.metadata.IPage
import com.baomidou.mybatisplus.core.toolkit.Constants
import org.apache.ibatis.annotations.Param
import java.io.Serializable

/**
 * Created by niantuo on 2018/9/29.
 * 实现物理操作的mapper基类
 */

interface IPhysicalMapper<T> {


    /**
     *
     *
     * 根据 entity 条件，删除记录
     *
     *
     * @param queryWrapper 实体对象封装操作类（可以为 null）
     */
    fun physicalDelete(@Param(Constants.WRAPPER) queryWrapper: Wrapper<T>): Int

    /**
     *
     *
     * 根据 ID 删除
     *
     *
     * @param id 主键ID
     */
    fun physicalDeleteById(id: Serializable): Int

    /**
     *
     *
     * 根据 columnMap 条件，删除记录
     *
     *
     * @param columnMap 表字段 map 对象
     */
    fun physicalDeleteByMap(@Param(Constants.COLUMN_MAP) columnMap: Map<String, Any>): Int



    /**
     *
     *
     * 删除（根据ID 批量删除）
     *
     *
     * @param idList 主键ID列表(不能为 null 以及 empty)
     */
    fun physicalDeleteBatchIds(@Param(Constants.COLLECTION) idList: Collection<Serializable>): Int

    /**
     *
     *
     * 根据 ID 修改
     *
     *
     * @param entity 实体对象
     */
    fun physicalUpdateById(@Param(Constants.ENTITY) entity: T): Int

    /**
     *
     *
     * 根据 whereEntity 条件，更新记录
     *
     *
     * @param entity        实体对象 (set 条件值,不能为 null)
     * @param updateWrapper 实体对象封装操作类（可以为 null,里面的 entity 用于生成 where 语句）
     */
    fun physicalUpdate(@Param(Constants.ENTITY) entity: T, @Param(Constants.WRAPPER) updateWrapper: Wrapper<T>): Int

    /**
     *
     *
     * 根据 ID 查询
     * 物理查询，忽略逻辑删除配置
     *
     * @param id 主键ID
     */
    fun physicalSelectById(id: Serializable): T


    /**
     *
     *
     * 根据 entity 条件，查询全部记录
     *
     *
     * @param queryWrapper 实体对象封装操作类（可以为 null）
     */
    fun physicalsSelectList(@Param(Constants.WRAPPER) queryWrapper: Wrapper<T>): List<T>


    /**
     *
     *
     * 根据 entity 条件，查询全部记录（并翻页）
     *
     *
     * @param page         分页查询条件（可以为 RowBounds.DEFAULT）
     * @param queryWrapper 实体对象封装操作类（可以为 null）
     */
    fun physicalSelectPage(page: IPage<T>, @Param(Constants.WRAPPER) queryWrapper: Wrapper<T>): IPage<T>
}