package cn.gygxzc.tina.orm.handler

import cn.gygxzc.tina.cloud.jwt.session.utils.SessionUtils
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler
import org.apache.ibatis.reflection.MetaObject
import org.slf4j.LoggerFactory
import java.util.*

/**
 * 公共字段自动填充
 * 多线程 + WebSocket 下不适用
 * @author NorthLan
 * @date 2018/1/31
 * @url https://noahlan.com
 */
class CustomMetaObjectHandler : MetaObjectHandler {
    companion object {
        val logger = LoggerFactory.getLogger(CustomMetaObjectHandler::class.java)!!
    }

    /**
     * 公告字段插入的时候填充数据
     */
    override fun insertFill(metaObject: MetaObject?) {
        try {
            if (getFieldValByName("createBy", metaObject) == null) {
                val curUser = SessionUtils.getUser()?.account
                setFieldValByName("createBy", curUser, metaObject)
            }
            if (getFieldValByName("createTime", metaObject) == null) {
                setFieldValByName("createTime", Date(), metaObject)
            }
        } catch (e: Throwable) {
            logger.warn("insertFill error, cause: ", e)
        }
    }

    override fun updateFill(metaObject: MetaObject?) {
        try {
            if (getFieldValByName("updateBy", metaObject) == null) {
                val curUser = SessionUtils.getUser()?.account
                setFieldValByName("updateBy", curUser, metaObject)
            }
            setFieldValByName("updateTime", Date(), metaObject)
        } catch (e: Throwable) {
            logger.warn("updateFill error, cause: ", e)
        }
    }
}