package com.gxzc.zen.common.exception.aop

import com.gxzc.zen.common.base.tips.ErrorTip
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import org.slf4j.LoggerFactory
import org.springframework.http.HttpStatus
import org.springframework.web.bind.annotation.ControllerAdvice
import org.springframework.web.bind.annotation.ExceptionHandler
import org.springframework.web.bind.annotation.ResponseBody
import org.springframework.web.bind.annotation.ResponseStatus

/**
 * 全局的的异常拦截器（拦截所有的控制器）（带有@RequestMapping注解的方法上都会拦截）
 * @author NorthLan
 * @date 2018/2/28
 * @url https://noahlan.com
 */
@ControllerAdvice
class BaseControllerExceptionHandler {
    companion object {
        private val logger = LoggerFactory.getLogger(BaseControllerExceptionHandler::class.java)
    }


    /**
     * 系统业务异常
     */
    @ExceptionHandler(ZenException::class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    @ResponseBody // Convert to Json
    fun zenException(e: ZenException): ErrorTip {
        logger.error("系统异常:", e)
        return ErrorTip(e.code, e.message)
    }

    /**
     * 未知的运行时异常
     */
    @ExceptionHandler(RuntimeException::class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    @ResponseBody
    fun runtimeException(e: RuntimeException): ErrorTip {
        logger.error("运行时异常:", e)
        return ErrorTip(ZenExceptionEnum.SERVER_ERROR.code, e.message ?: "")
    }
}