package com.gxzc.zen.common.util

import org.slf4j.LoggerFactory
import org.springframework.beans.BeansException
import org.springframework.context.ApplicationContext
import org.springframework.context.ApplicationContextAware
import org.springframework.stereotype.Component

/**
 * Spring Context Holder
 * @author NorthLan
 * @date 2018/2/1
 * @url https://noahlan.com
 */
@Component
class SpringContextHolder : ApplicationContextAware {
    companion object {
        private val logger = LoggerFactory.getLogger(SpringContextHolder::class.java)
        private lateinit var applicationContext: ApplicationContext

        @Suppress("UNCHECKED_CAST")
        fun <T> getBean(beanName: String): T? {
            return try {
                applicationContext.getBean(beanName) as? T
            } catch (e: BeansException) {
                logger.warn("get bean [$beanName] error: ${e.message}")
                null
            }
        }

        fun <T : Any> getBean(requiredType: Class<T>): T? {
            return try {
                applicationContext.getBean(requiredType)
            } catch (e: BeansException) {
                logger.warn("get bean [${requiredType.name}] error: ${e.message}")
                null
            }
        }
    }

    init {
        logger.info("${this::class.java.simpleName} initializing...")
    }

    override fun setApplicationContext(applicationContext: ApplicationContext) {
        SpringContextHolder.applicationContext = applicationContext
    }
}